##############################################################################
#
# Copyright (c) 2003 gocept gmbh & co. kg. All rights reserved.
#
# See also LICENSE.txt
#
##############################################################################
"""CMF link checker tool - link information object

$Id: LinkInfo.py,v 1.3 2003/11/15 17:32:53 ctheune Exp $"""

from Globals import Persistent
from Acquisition import Implicit
from OFS.SimpleItem import SimpleItem
from interfaces import ILinkInformation
from urllib import urlencode
from Zope import app

from Products.CMFCore.utils import getToolByName
from httpcodes import http_codes

app_ = None

class LinkInfo(SimpleItem):

    __implements__ = (ILinkInformation, )
    __allow_access_to_unprotected_subobjects__ = 1      # XXX make accessor functions

    owner = None
    object = None
    url = None
    lastcheck = None
    lasttime = None
    queued = 0
    state = "unknown"
    state_class = "alert"
    headers = {}
    reason = ""

    def getCacheKey(self):
        """Construct a key that's good for identifying this link.
        
        Returns a key that is usable as a query string and a unique identifier
        for this object."""
        return str(tuple(self.object) + (self.url, ))

    def getObject(self):
        """Return a reference to the object."""
        object = self.__getApp().unrestrictedTraverse(self.object)
        return object

    def __getApp(self):
        global app_
        if not app_:
            app_ = app()
        return app_

    def setUrl(self, url, REQUEST=None):
        """Set the URL.

        Sets the URL to the given value, trying to make relative paths absolute.
        (Requires a REQUEST object)"""
        self.url = url

        if REQUEST is None:
            return

        # Try to normalize relative links
        oldurl = self.url
        if self.url.startswith("/"):
            # Maybe we have a default prefix on the tool?
            lt = getToolByName(self, 'portal_linkchecker', None)
            self.url = getattr(lt, 'defaultURIPrefix', REQUEST.BASE0) + self.url
        elif not ':' in self.url:
            try:
                self.url = REQUEST.BASE0+'/'.join(self.object)+'/'+self.url
            except:
                pass

    def getGroupId(self):
        return http_codes.get(self.state, [0, 'unknown'])[1]
