##############################################################################
#
# Copyright (c) 2003 gocept gmbh & co. kg. All rights reserved.
#
# See also LICENSE.txt
#
##############################################################################
"""CMF link checker tool interface definitions

$Id: interfaces.py,v 1.2 2003/11/15 15:23:37 ctheune Exp $"""
    
from Interface import Interface, Attribute

class ILinkCheckerTool(Interface):

    maximumCacheAge = Attribute("Integer that defines the age of valid information in seconds.")
    gracefulUpdateAge = Attribute("Integer that defines the age when an update is desirable.")

    def registerRetriever(portal_type, function_ref):
        """Registers the function_ref to be called on objects
           of portal_type.

           The signature of function_ref must be accepting a single object
           which is the context. The return value is a list of
           ILinkInformation objects.  """

    def checkLink(url):
        """Checks the given URL. 
        
        This result may be cached to allow intelligent load balancing for the
        blocking calls. Returns a boolean value which means true for a link
        that is online and false for a link that is offline."""

    def showLinks():
        """Show the information about the known links.

        Doesn't trigger updates or anything."""

    def checkAllLinks():
        """Schedules checks for all links.

        Takes all links an puts them in the queue."""

    def findLinks():
        """Find new links in the site.

        Finds new links from all objects that are retrievable and
        store them in the cache. No link check is done here."""
        
    def setMaximumCacheAge(age):
        """Sets the maximum cache age."""

    def setGracefulUpdateAge(age):
        """Sets the graceful update age.

        Assures that the graceful update age is lower than the
        maximum cache age."""
        
    def setDefaultURIPrefix(defaultURIPrefix):
        """Set the default URI prefix that will be appended to
           relative URLs.

           If set to a <false> value, it will prepend REQUEST.URL0."""

    def activateWorkerThread():
        """Activate the worker thread.

        Starts the worker thread (if it's not running already)."""
        
class ILinkInformation(Interface):
    """Information about a link."""

    owner = Attribute("Contains a CMFMember who is the owner of the object.")
    object = Attribute("Contains the object the link came from.")
    url = Attribute("Contains the URL the link refers to.")
    lastcheck = Attribute("DateTime when the last check was performed.") 
    lasttime = Attribute("Time (in seconds) the last check took.")
    state = Attribute("A HTTP Response Code.")
    reason = Attribute("A verbose reason describing the error")
    headers = Attribute("Dictionary of the headers of the last response.")

    def __init__(object, owner, url):
        """Constructor."""
