##############################################################################
#
# Copyright (c) 2003 gocept gmbh & co. kg. All rights reserved.
#
# See also LICENSE.txt
#
##############################################################################
"""CMF link checker tool - link retriever functions

$Id: retrievers.py,v 1.3 2003/11/15 16:49:47 ctheune Exp $"""

from Products.CMFDefault.Link import Link
from Products.CMFDefault.Document import Document
from Products.CMFCalendar.Event import Event

from StructuredText import Basic
from StructuredText.DocumentClass import StructuredTextLink, DocumentClass

from LinkInfo import LinkInfo

import htmllib
from sgmllib import SGMLParseError
import formatter

def retrieverLink(object):
    """Usable for CMFDefault.Link objects."""
    assert isinstance(object, Link)
    info = LinkInfo()
    info.object = object.getPhysicalPath()
    info.setUrl(object.getRemoteUrl())
    return [info]
    
def retrieverEvent(object):
    """Usable for CMFDefault.Event objects."""
    assert isinstance(object, Event)
    info = LinkInfo()
    info.object = object.getPhysicalPath()
    info.setUrl(object.event_url)
    return [info]
    
def retrieveDocument(object):
    """Usable for CMFDefault.Document objects (in HTML and STX mode)"""
    assert isinstance(object, Document)
    
    if object.text_format == 'structured-text':
        links = retrieveSTX(object.text)
        links = [link.href for link in links]
    elif object.text_format == 'html':
        links = retrieveHTML(object.text)
    else:
        links = []

    retrieved = []
    for link in links:
        info = LinkInfo()
        info.object = object.getPhysicalPath()
        info.setUrl(link)
        retrieved.append(info)

    return retrieved

def retrieveSTX(text):
    """Retrieve links from STX data."""
    data = Basic(text)
    data = DocumentClass()(data)
    
    stack = data.getChildren()
    links = []
    while stack:
        element = stack.pop()
        if isinstance(element, StructuredTextLink):
            links.append(element)
        try:
            stack.extend(element.getChildren())
        except AttributeError:
            pass

    return links

def retrieveHTML(text):
    parser = htmllib.HTMLParser(formatter.NullFormatter())
    try:
        parser.feed(text)
    except SGMLParseError:
        # SGMLLib seems to die on bad HTML sometimes. (At least with python2.1)
        pass
    parser.close()
    return tuple(parser.anchorlist)

# simple registries

retrievers = { 
      'Link'     :   retrieverLink,
      'Event'    :   retrieverEvent,
      'Favorite' :   retrieverLink,
      'Document' :   retrieveDocument,
      'News Item' :  retrieveDocument }
